==> harbour-1.0.1-src.zip: harbour-1.0.1/contrib/hbct/misc2.c <==
/*
 *  $Id: misc2.c 7630 2007-08-02 15:45:52Z druzus $
 */

/*
 * Harbour Project source code:
 *   CT3 Miscellaneous functions: - COMPLEMENT(), NUL()
 *
 * Copyright 2005 Pavel Tsarenko <tpe2@mail.ru>
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 * As a special exception, the Harbour Project gives permission for
 * additional uses of the text contained in its release of Harbour.
 *
 * The exception is that, if you link the Harbour libraries with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the Harbour library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the Harbour
 * Project under the name Harbour.  If you copy code from other
 * Harbour Project or Free Software Foundation releases into a copy of
 * Harbour, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for Harbour, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 *
 */

#include "hbapi.h"
#include "hbapiitm.h"

HB_FUNC( COMPLEMENT )
{
   PHB_ITEM pItem = hb_param( 1, HB_IT_ANY );

   if( pItem )
   {
      if( HB_IS_STRING( pItem ) )
      {
         ULONG ulLen = hb_itemGetCLen( pItem ), ulPos;

         if( ulLen > 0 )
         {
            char *szBuffer = ( char * ) hb_xgrab( ulLen + 1 ), *szSrc = hb_itemGetCPtr( pItem );

            for( ulPos = 0; ulPos < ulLen; ulPos++ )
               szBuffer[ulPos] = ~szSrc[ulPos];
            hb_retclen_buffer( szBuffer, ulLen );
         }
         else
            hb_retc( NULL );
      }
      else if( HB_IS_DATE( pItem ) )
         hb_retdl( 4537847 - hb_itemGetDL( pItem ) );
      else if( HB_IS_NUMINT( pItem ) )
         hb_retnint( -hb_itemGetNInt( pItem ) );
      else if( HB_IS_NUMERIC( pItem ) )
      {
         int iWidth, iDec;
         double dValue;

         dValue = hb_itemGetND( pItem );
         hb_itemGetNLen( pItem, &iWidth, &iDec );
         hb_retndlen( -dValue, iWidth, iDec );
      }
      else if( HB_IS_LOGICAL( pItem ) )
         hb_retl( !hb_itemGetL( pItem ) );
      else
         hb_ret();
   }
   else
      hb_ret();
}


HB_FUNC( NUL )
{
   hb_retc( NULL );
}

==> harbour-2.0.0-src.zip: harbour-2.0.0/contrib/hbct/misc2.c <==
/*
 *  $Id: misc2.c 11616 2009-07-03 07:56:51Z vszakats $
 */

/*
 * Harbour Project source code:
 *   CT3 Miscellaneous functions: - COMPLEMENT(), NUL()
 *
 * Copyright 2005 Pavel Tsarenko <tpe2@mail.ru>
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 * As a special exception, the Harbour Project gives permission for
 * additional uses of the text contained in its release of Harbour.
 *
 * The exception is that, if you link the Harbour libraries with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the Harbour library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the Harbour
 * Project under the name Harbour.  If you copy code from other
 * Harbour Project or Free Software Foundation releases into a copy of
 * Harbour, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for Harbour, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 *
 */

#include "hbapi.h"
#include "hbapiitm.h"

HB_FUNC( COMPLEMENT )
{
   PHB_ITEM pItem = hb_param( 1, HB_IT_ANY );

   if( pItem )
   {
      if( HB_IS_STRING( pItem ) )
      {
         ULONG ulLen = hb_itemGetCLen( pItem ), ulPos;

         if( ulLen > 0 )
         {
            const char *szSrc = hb_itemGetCPtr( pItem );
            char *szBuffer = ( char * ) hb_xgrab( ulLen + 1 );

            for( ulPos = 0; ulPos < ulLen; ulPos++ )
               szBuffer[ulPos] = ~szSrc[ulPos];
            hb_retclen_buffer( szBuffer, ulLen );
         }
         else
            hb_retc_null();
      }
      else if( HB_IS_DATE( pItem ) )
         hb_retdl( 4537847 - hb_itemGetDL( pItem ) );
      else if( HB_IS_TIMESTAMP( pItem ) )
         hb_rettd( 4537847.0 - hb_itemGetTD( pItem ) );
      else if( HB_IS_NUMINT( pItem ) )
         hb_retnint( -hb_itemGetNInt( pItem ) );
      else if( HB_IS_NUMERIC( pItem ) )
      {
         int iWidth, iDec;
         double dValue;

         dValue = hb_itemGetND( pItem );
         hb_itemGetNLen( pItem, &iWidth, &iDec );
         hb_retndlen( -dValue, iWidth, iDec );
      }
      else if( HB_IS_LOGICAL( pItem ) )
         hb_retl( !hb_itemGetL( pItem ) );
      else
         hb_ret();
   }
   else
      hb_ret();
}


HB_FUNC( NUL )
{
   hb_retc_null();
}
